
DrawSprite
	;HL - screen position
	;DE - address
	;BC -size (width,height)
	
DrawSprite2	
	PUSH HL
	PUSH BC
	
DrawSprite3
	;one row of characters			
	
	PUSH BC
	LD B,8
	LD C,H
	
DrawSprite4		
	;one character
	LD A,(DE)
	LD (HL),A
	INC H				;next line on screen
	INC DE				;next data byte
	DJNZ DrawSprite4
	
	LD H,C				;restore top line of characters
	INC L				;next character right
	POP BC
	DJNZ DrawSprite3
	
	POP BC
	POP HL				;go to leftmost character
	CALL OneCharDown	;next row
	DEC C
	JR NZ,DrawSprite2
	RET	
	
;-------------
DrawEmptySprite
	LD DE,EmptySprite			;clear area
	CALL DrawSprite
	RET
	

;-----------
OneCharDown
	LD A,L
	ADD A,32
	LD L,A
	RET NC				;stay in the same screen segment
	
	LD A,H				;next screen segment
	ADD A,8
	LD H,A
	RET
	
;-----------
OneCharUp
	LD A,L
	SUB 32
	LD L,A
	RET NC				;stay in the same screen segment
	
	LD A,H				;next screen segment
	SUB 8
	LD H,A
	RET

;-----------
GraphToAttr
;graphics char HL to attribute char HL	
	PUSH AF
	LD A,80
	SRL H
	SRL H
	SRL H
	ADD A,H
	LD H,A
	POP AF
	RET

;----------
AttrBlock
;paint area with attribute
;HL - corner, BC - size, A- attribute
	LD DE,32
	
AttrBlock2	
	PUSH HL
	PUSH BC
;one line	
AttrBlock3
	LD (HL),A
	INC L
	DJNZ AttrBlock3
	
	POP BC
	POP HL
	ADD HL,DE			;next line
	DEC C
	JR NZ,AttrBlock2
	RET	
;-----------
ClearScreen
	XOR A
	LD (23624),A 
	OUT (254),A
	
	LD HL,16384
	LD (HL),A
	LD DE,16385
	LD BC,6143
	LDIR

	INC HL
	INC DE
	LD A,71	
	LD (HL),A
	LD BC,767
	LDIR

	RET
	

;-----------------------
SetBoard	
;find current level data
	LD HL,Levels-32
	LD DE,32
	LD A,(LevelNr)
SetBoard1
	ADD HL,DE
	DEC A
	JR NZ,SetBoard1
	EX HL,DE				;level data into DE		


	LD HL,BoardData
	
	LD B,32	
SetBoard2	
;1st marble		
	LD A,(DE)				;read byte of level data
	AND 11110000b
	RRA
	RRA
	RRA
	RRA						;get high 4 bits	(1 byte codes two marbles)
	LD (HL),A				;write marble number into board data
	INC HL					;next byte in board data
	
;2nd marble	
	LD A,(DE)
	AND 00001111b
	LD (HL),A				;write marble number into board data

	INC HL					;next byte in board data
	INC DE					;next byte of data
	DJNZ SetBoard2			;do 32*2 = 64 marbles
	RET
	
	
;-----------------------	
DrawBoard
	LD DE,BoardData
	LD HL, 16384+4			;left upper corner of board area
	
	LD C,8					;8 rows
	
DrawBoard2	
	PUSH HL
	LD B,8					;8 marbles in a row
	
DrawBoard3
	PUSH BC
	
;1st marble		
	LD A,(DE)				;read marble type from of level data
	
	
	PUSH HL
	PUSH DE
	CALL DrawPiece			;draw marble
	POP DE
	POP HL
	
	INC L					;go to next marble position
	INC L
	INC L
	
	INC DE					;next byte of data
	
	POP BC
	DJNZ DrawBoard3			;complete one row 
	
	POP HL
	CALL OneCharDown		;go to next row
	CALL OneCharDown
	CALL OneCharDown
	
	DEC C					;8 rows
	JR NZ,DrawBoard2
	RET

;---------
DrawPiece
	;A - number of piece,HL-position
	OR A				
	JR NZ,DrawPiece1		;not empty field
	
	LD BC,3*256+3			;size
	CALL DrawEmptySprite	;clear area
	RET
	
	
	
DrawPiece1	
	LD DE,Marble
	CP 8
	JR C,DrawPiece2		;choose graphics
	LD DE,Diamond
	
	
DrawPiece2
	PUSH HL
	PUSH AF
	LD BC,3*256+3			;size
	CALL DrawSprite
	POP AF
	POP HL
	
;paint attributes
	CP 8					
	JR C,DrawPiece3			;number of marble = its color for normal marbles
	SUB 8					;color=nr-8 for diamonds

DrawPiece3
	CALL GraphToAttr
	LD BC,3*256+3			;size
	CALL AttrBlock	

	RET	
	
	
;---------------
TransformBoardCoords
;board position in BC (0-7,0-7) to HL screen position	
;put into A nr of marble at BC


;find position of left side of row	
	LD HL,16384+4
	
	EXX
	LD HL,BoardData				;for counting marble nr
	LD DE,8
	EXX
	
	
	
	LD A,B
	OR A
	JR Z,TransformBoardCoords4

TransformBoardCoords2	
	EXX
	ADD HL,DE					;counting position in table, adding rows
	EXX
	
	LD A,L						;go 3 char lines down
	ADD A,96
	LD L,A
	JR NC,TransformBoardCoords3
	
	LD A,H						;next screen segment
	ADD A,8
	LD H,A

TransformBoardCoords3
	DJNZ TransformBoardCoords2


TransformBoardCoords4
;find proper collumn in the row
	LD A,C
	OR A
	JR Z,TransformBoardCoords6		;no need to adjust for collumn 0
	
TransformBoardCoords5
	INC L
	INC L
	INC L
	DEC C
	
	EXX
	INC HL						;counting position in table, adding cols
	EXX
	
	JR NZ,TransformBoardCoords5
	
TransformBoardCoords6
	EXX
	LD A,(HL)					;get marble nr
	EXX
	
	RET
	
		

;---------------
DrawCollumns
	LD HL,16384
	CALL DrawCollumn
	LD HL,16384+28
	CALL DrawCollumn
	RET
	
DrawCollumn
;HL-top left corner
;draw collumn top
	LD DE,Collumn1
	LD BC,4*256+3
	CALL DrawSprite

;draw collumn trunk
 	LD B,20					;char rows
 
DrawCollumn2	
 	PUSH BC
	PUSH DE					;store graphics (already set to next part of collumn)
 	PUSH HL					;store left screen corner
 	
 	LD BC,4*256+1			;size
 	CALL DrawSprite
 	POP HL
 	POP DE
 	POP BC
 	CALL OneCharDown
 	DJNZ DrawCollumn2	

;draw collumn bottom
	LD DE,Collumn3
	LD BC,4*256+1
	CALL DrawSprite
	RET
	
;------------------------
DrawPointer	
	LD BC,(PointerPos)
	CALL TransformBoardCoords
	
	LD DE,Pointer
	CALL DrawPointer_OneArrow	;LU arrow
	INC L
	INC L
	CALL DrawPointer_OneArrow	;RU arrow
	CALL OneCharDown
	CALL OneCharDown
	CALL DrawPointer_OneArrow	;RD arrow
	DEC L
	DEC L
	CALL DrawPointer_OneArrow	;LD arrow
	RET
	
	

;----
DrawPointer_OneArrow	
	
	LD BC,1*256+1
	PUSH HL
	CALL DrawSprite
	POP HL
	
	PUSH HL
	CALL GraphToAttr
	LD A,(PointerColor)
	LD (HL),A					;paint attribute
	POP HL
	RET
	
	
;------------------------
Collumn1:
	.BYTE	127, 63, 31,  0, 30, 63, 96,222
	.BYTE	255,255,250,  0, 63,135,240, 63
	.BYTE	254,213,170,  0,252,225, 15,252
	.BYTE	162, 68,136,  0,120,252,  6,123
	.BYTE	179,173,189,217, 98, 60,  0, 31
	.BYTE	  7, 64,120,127,127,245,  0,255
	.BYTE	224,  2, 30,244,170, 85,  0,255
	.BYTE	205,181,189,155, 70, 60,  0,240
	.BYTE	 63, 31,  0,  0,  5,  0,  5,  0
	.BYTE	255,255,  0,  0,127,175,127,175
	.BYTE	255,255,  0,  0,254,245,254,245
	.BYTE	248,240,  0,  0,160,  0,160,  0

Collumn2:
	.BYTE	  5,  0,  5,  0,  5,  0,  5,  0
	.BYTE	127,175,127,175,127,175,127,175
	.BYTE	254,245,254,245,254,245,254,245
	.BYTE	160,  0,160,  0,160,  0,160,  0

Collumn3:
	.BYTE	  0,  7,  7, 15,  0, 31, 63,127
	.BYTE	  0,255,255,255,  0,255,255,255
	.BYTE	  0,255,255,255,  0,255,234, 85
	.BYTE	  0, 32, 32,144,  0, 68,162, 81	
	
Marble:
	.BYTE	  0,  0,  1,  7, 12, 24, 16, 48
	.BYTE	  0,126,195,  0,  0, 10, 85,170
	.BYTE	  0,  0,128,224, 48, 24, 72,164
	.BYTE	 33, 98, 65, 66, 69, 66, 69, 35
	.BYTE	 87,191,127,255,255,255,255,255
	.BYTE	244,254,250,254,254,250,254,252
	.BYTE	 37, 19, 19, 11,  7,  1,  0,  0
	.BYTE	255,255,255,255,255,255,126,  0
	.BYTE	252,248,248,240,224,128,  0,  0
	
Diamond:
	.BYTE	  0,  0,  0,  0,  0,  1,  2,  4
	.BYTE	  0, 24, 44, 70,131,  1, 80,160
	.BYTE	  0,  0,  0,  0,  0,128,192, 96
	.BYTE	  9, 18, 37, 64, 64, 32, 16,  8
	.BYTE	 64,128,  0,  0,  1,  2,  5, 42
	.BYTE	 48, 56, 92,174, 86,172, 88,240
	.BYTE	  5,  2,  1,  0,  0,  0,  0,  0
	.BYTE	 85,171, 95,255,126, 60, 24,  0
	.BYTE	224,192,128,  0,  0,  0,  0,  0
	
DestroyedMarble	
	.BYTE	  0,  0,  5,  0, 21,  0, 21,  0
	.BYTE	 85,  0, 85,  0, 85,  0, 85,  0
	.BYTE	  0,  0, 80,  0, 84,  0, 84,  0
	.BYTE	 85,  0, 85,  0, 85,  0, 85,  0
	.BYTE	 85,  0, 85,  0, 85,  0, 85,  0
	.BYTE	 85,  0, 85,  0, 85,  0, 85,  0
	.BYTE	 21,  0, 21,  0,  5,  0,  0,  0
	.BYTE	 85,  0, 85,  0, 85,  0, 85,  0
	.BYTE	 84,  0, 84,  0, 80,  0,  0,  0	
	

Pointer
	.BYTE	 64,226,118, 46, 30, 62,126,  0
	.BYTE	  2, 71,110,116,120,124,126,  0
	.BYTE	  0,126,124,120,116,110, 71,  2	
	.BYTE	  0,126, 62, 30, 46,118,226, 64
	
Items:
	.BYTE	  0,  1,  2,  4,  9, 18, 37, 64
	.BYTE	  0,255,  1,249,245,169, 85,  1
	.BYTE	127,  0,  1,  1,  1,  1,  1,  1
	.BYTE	255,  0,208,224,208,224,208,224
	.BYTE	  1,  1,  1,  1,  1,  1,  1,  0
	.BYTE	208,224,208,224,208,224,208,224
	.BYTE	 33,115,115,  0, 33, 33, 33, 49
	.BYTE	  8,156,156,  0,  8,  8,  8, 24
	.BYTE	 27, 12,  5,  4,  3,  3,  0,  3
	.BYTE	176, 96, 64, 64,128,128,  0,128
	.BYTE	  3,  0,  3,  3,  0,  3,  3,  1
	.BYTE	128,  0,128,128,  0,128,128,  0
	.BYTE	  3,  6, 12,  8, 24, 17, 18, 16
	.BYTE	192, 96, 48, 16,152,  8, 72,168
	.BYTE	 17, 26,  8, 12,  6,  3,  1,  1
	.BYTE	 72,152, 16, 48, 96,192,128,128
	.BYTE	  1,  3,  2,  5,  5,  2,  3,  1
	.BYTE	128,192, 64,160,160, 64,192,128
	.BYTE	  0,  1,  1,  3,  3,  3,  3,  3
	.BYTE	128, 64, 64, 96, 96, 96, 96, 96
	.BYTE	  3,  3,  3,  3,  3,  3,  3,  0
	.BYTE	 96, 96, 96, 96, 96, 96, 96,  0
	.BYTE	  7,  0,  1,  0,  1,  0,  1,  0
	.BYTE	240,  0,192,  0,192,  0,192,128
	.BYTE	  0,  1,  3,  7, 15, 31, 63, 30
	.BYTE	252,248,240,224,192,128,  0,  0
	.BYTE	 15,  7,  3,  1,  0,  0,  0,  0
	.BYTE	  0,128,192,224,240,120, 56,112
	.BYTE	  0,  1,  3,  0, 31, 15,  7,  2
	.BYTE	224,192,128,  0,192,128,  0,  0
	
ItemColors	.BYTE 66,65,69,71,70

Girl
	.BYTE	  0, 13, 18, 32, 67, 69, 39, 22
	.BYTE	192, 32, 16,  8,136, 72,208,192
	.BYTE	  3,  3, 31, 63,127,126,238,209
	.BYTE	128,128,240,248,252,252,238, 22
	.BYTE	223,207,207,207,231,103,  7, 15
	.BYTE	246,246,230,230,238,172, 32,160
	.BYTE	 15, 14, 31, 30, 29, 30, 28, 31
	.BYTE	 16,144, 16, 16, 16, 16, 16,240		
	
;-------------
Sound1
	LD B,3
	LD HL, 100				;initial frequency
	
Sound12
	PUSH BC
	PUSH HL
	LD DE,25					;length
	CALL 949
	POP HL
	LD DE,100
	ADD HL,DE
	
	POP BC
	DJNZ Sound12
	RET
	
;---------
Sound2
	LD HL, 2000				;initial frequency
	LD DE,100					;length
	CALL 949
	RET
	
;---------
Sound3
	LD B,3
	LD HL, 2100				;initial frequency
	
Sound32
	PUSH BC
	PUSH HL
	LD DE,5					;length
	CALL 949
	POP HL
	LD DE,100
	SUB HL,DE
	
	POP BC
	DJNZ Sound32
	RET
	
;---------
ExplosionSound
	LD L,6
	LD E,0
	LD D,64
Explosion1	
	PUSH DE
Explosion2	
	LD B,E
Explosion3
	DJNZ Explosion3
	LD A,(BC)
	AND 248
	OR L
	AND 11111000B
	OUT (254),A
	INC C
	DEC D
	JR NZ,Explosion2
	
	POP DE
	INC E
	DEC D
	JR NZ,Explosion1		
	RET
	
Sound4
	LD B,40
	LD HL, 100				;initial frequency
	
Sound41
	PUSH BC
	PUSH HL
	LD DE,10					;length
	CALL 949
	POP HL
	LD DE,20
	ADD HL,DE
	
	POP BC
	DJNZ Sound41
	RET