

;----------------------
MovePointer
	LD A,(MarbleSelected)		;if player pressed fire before
	OR A						;don't move pointer but try
	JR NZ,SwapMarbles			;to move selected marble

	LD HL,(PointerPos)
	LD (OldPointerPos),HL		;store old position
	
	CALL GetPlayerInput
	LD HL,PointerPos
	
MoveLeft
	BIT 1,A
	JR Z,MoveRight
	
	LD D,A						;store and restore byte with movement bits
	LD A,(HL)
	OR A
	LD A,D
	JR Z,MoveRight				;board edge
	DEC (HL)					;move right
	JR MovePointerEnd

MoveRight
	BIT 0,A
	JR Z,MoveUp
	
	LD D,A
	LD A,(HL)
	CP 7
	LD A,D
	JR Z,MoveUp				;board edge
	INC (HL)
	JR MovePointerEnd


MoveUp
	INC HL					;go to Y cooridnate
	BIT 3,A

	JR Z,MoveDown
	
	LD D,A
	LD A,(HL)
	OR A
	LD A,D
	JR Z,MoveDown				;board edge
	DEC (HL)
	JR MovePointerEnd

MoveDown
	BIT 2,A
	JR Z,PressFire		
	
	LD D,A
	LD A,(HL)
	CP 7
	LD A,D
	JR Z,PressFire		;board edge
	INC (HL)
	JR MovePointerEnd
	
	
PressFire
	BIT 4,A					;player pressed fire?
	JR Z,MovePointerEnd2	;no, so he didn't press anything, don't play sound and finish
	LD (MarbleSelected),A	;mark that fire was pressed, marble is selected and ready for swap
	RET

	
MovePointerEnd
;play sound when the pointer moved
	CALL Sound1
	HALT
	HALT

MovePointerEnd2
;drawing marble at old position and pointer at new position
	LD BC,(OldPointerPos)
	CALL TransformBoardCoords	;HL-screen position,A - marble nr
	CALL DrawPiece				;marble at old position
	
	CALL DrawPointer			;pointer at new position
	RET
	
	
	
	
;--------------------------------------
SwapMarbles
	CALL GetPlayerInput
	BIT 4,A
	JR NZ,SwapMarbles2
	
	XOR A						;player released fire;
	LD (MarbleSelected),A
	
SwapMarbles2
;pointer animation
	PUSH AF
	HALT
	LD A,66
	LD (PointerColor),A
	CALL DrawPointer
	HALT
	LD A,71
	LD (PointerColor),A
	CALL DrawPointer
	POP AF
	
	LD BC,(PointerPos)

;------------------		
SwapMarbles_Right
	BIT 0,A	
	JR Z,SwapMarbles_Left
	LD A,C
	CP 7
	RET Z						;board edge - no swap
	
	PUSH BC		
	CALL SwapLeftRight
	CALL FindMarbleChains		;any chains created after swap?
	
	
	LD A,(ChainExisting)
	OR A
	JP NZ,DestroyMarbles
	
	;no chains - undo move
	CALL Sound2
	POP BC
	CALL SwapLeftRight1			;make swap to return previous position
	RET							;(skip no empty space condition)
	
	
	
;------------------	
SwapMarbles_Left
	BIT 1,A	
	JR Z,SwapMarbles_Up
	LD A,C
	OR A
	RET Z						;board edge - no swap
	
	DEC C						;pointer pointed to right marbles of the pair
								;and we need the left one
	
	PUSH BC		
	CALL SwapLeftRight
	CALL FindMarbleChains		;any chains created after swap?
	
	
	LD A,(ChainExisting)
	OR A
	JP NZ,DestroyMarbles
	
	;no chains - undo move
	CALL Sound2
	POP BC
	CALL SwapLeftRight1			;make swap to return previous position
	RET							;(skip no empty space condition)
	

	
;------------------	
SwapMarbles_Up
	BIT 3,A	
	JR Z,SwapMarbles_Down
	LD A,B
	OR A
	RET Z						;board edge - no swap
	
	DEC B						;point to the top one of swapped marble pair
	
	PUSH BC		
	CALL SwapUpDown
	CALL FindMarbleChains		;any chains created after swap?
	
	
	LD A,(ChainExisting)
	OR A
	JP NZ,DestroyMarbles
	
	;no chains - undo move
	CALL Sound2
	POP BC
	CALL SwapUpDown1			;make swap to return previous position
	RET							;(skip no empty space condition)
	
	
;-----------------	
SwapMarbles_Down
	BIT 2,A	
	RET Z
	LD A,B
	CP 7
	RET Z						;board edge - no swap
	
	PUSH BC		
	CALL SwapUpDown
	CALL FindMarbleChains		;any chains created after swap?
	
	
	LD A,(ChainExisting)
	OR A
	JP NZ,DestroyMarbles
	
	;no chains - undo move
	CALL Sound2
	POP BC
	CALL SwapUpDown1			;make swap to return previous position
	RET							;(skip no empty space condition)


	
;------------------------------------
SwapLeftRight
	PUSH BC
	LD BC,(PointerPos)
	CALL TransformBoardCoords
	POP BC
	OR A						;A - nr of marble at pointer
	RET Z						;empty space at pointer = no swap


SwapLeftRight1
;B - X, C- Y of the left from two marbles being swapped
	CALL TransformBoardCoords				;HL - corner of swapped area
	LD B,4									;nr of repeats
	
	LD (Marble1Pos),HL						;initial positions of marbles
	
	PUSH HL
	LD A,L
	ADD A,3
	LD L,A	
	LD (Marble2Pos),HL
	POP HL									;restore corner of swapped area
	
	
SwapLeftRight2
;animation
	HALT
	HALT
	HALT
	
	PUSH BC									;nr of repeats
	PUSH HL									;store corner of swapped area
	LD BC,6*256+3
	CALL DrawEmptySprite
	
;draw left marble	
	LD HL,(Marble1Pos)
	EXX										;board data in alternate HL
	LD A,(HL)								;marble nr
	EXX

	CALL DrawPiece
	
;draw right marble	
	LD HL,(Marble2Pos)
	EXX
	INC HL
	LD A,(HL)								;marble nr
	DEC HL
	EXX
	
	OR A
	JR Z,SwapLeftRight3						;don't draw empty space this time
	CALL DrawPiece
	
	
SwapLeftRight3
;change marbles positions
	LD HL,Marble1Pos
	INC (HL)
	LD HL,Marble2Pos
	DEC (HL)
	
	POP HL
	POP BC
	DJNZ SwapLeftRight2
	

;swap marble numbers in the board data
	EXX
	LD B,(HL)
	INC HL
	LD C,(HL)
	LD (HL),B
	DEC HL
	LD (HL),C
	RET
	
	
;---------------------
SwapUpDown
	PUSH BC
	LD BC,(PointerPos)
	CALL TransformBoardCoords
	POP BC
	OR A						;A - nr of marble at pointer
	RET Z						;empty space at pointer = no spap

SwapUpDown1
; B - X, C- Y of the left from two marbles being swapped
	CALL TransformBoardCoords				;HL - corner of swapped area
	LD B,4									;nr of repeats
	
	LD (Marble1Pos),HL						;initial positions of marbles
	
	PUSH HL
	CALL OneCharDown	
	CALL OneCharDown	
	CALL OneCharDown	
	LD (Marble2Pos),HL
	POP HL									;restore corner of swapped area
	
	
SwapUpDown2
;animation
	HALT
	HALT
	HALT
	
	PUSH BC									;nr of repeats
	PUSH HL									;store corner of swapped area
	LD BC,3*256+6
	CALL DrawEmptySprite
	
;draw top marble	
	LD HL,(Marble1Pos)
	EXX										;board data in alternate HL
	LD A,(HL)								;marble nr
	EXX

	CALL DrawPiece
	
;draw bottom marble	
	LD HL,(Marble2Pos)
	EXX
	LD DE,8									;row is 8 marbles wide, one row down -> pos=pos+8
	ADD HL,DE
	LD A,(HL)								;marble nr
	SBC HL,DE
	
	EXX
	
	OR A
	JR Z,SwapUpDown3						;don't draw empty space this time
	
	CALL DrawPiece
	
	
SwapUpDown3
;change marbles positions
	LD HL,(Marble1Pos)
	CALL OneCharDown
	LD (Marble1Pos),HL
	
	LD HL,(Marble2Pos)
	CALL OneCharUp
	LD (Marble2Pos),HL
	
	POP HL
	POP BC
	DJNZ SwapUpDown2
	

;swap marble numbers in the board data
	EXX
	LD DE,8
	LD B,(HL)
	ADD HL,DE
	LD C,(HL)
	LD (HL),B
	SUB HL,DE
	LD (HL),C
	RET

	
Marble1Pos		.WORD 0
Marble2Pos		.WORD 0
	
	

;--------------------------------------	
GetPlayerInput
	LD A,(KeysOrJoystick)
	OR A						;keyboard or joystick
	JR Z,KeyboardControl
	
JoystickControl
	LD BC,31
	IN A,(C)
	RET
	
KeyboardControl
;the aim is to set bits of A register as it would be joystick input
	LD D,0
	LD BC,57342; O,P	
	IN A,(C)
	CPL
	AND 00000011B
	LD D,A
	LD BC,64510;Q
	IN A,(C)
	CPL
	AND 1
	RLA
	RLA
	RLA
	OR D		
	LD D,A		
	LD BC,65022;A
	IN A,(C)
	CPL
	AND 1
	RLA
	RLA
	OR D
	LD D,A
	LD BC,32766 ;space
	IN A,(C)
	CPL
	AND 1
	RLA
	RLA
	RLA
	RLA
	OR D ;ready
	RET

KeysOrJoystick .BYTE 0
