DestroyMarbles
	CALL AnimateDestruction	
	CALL Explosions
		
	CALL EraseMarbles					;erase marbles from level data
	CALL DrawBoard
	HALT

	CALL Falling						;falling marbles in collumns after destruction
	
	CALL FallingAnimation				;copy new data after falling without animation
	
;after marbles falling there could be a new row of 3 or more marbles
;so check for chain reaction	

	CALL FindMarbleChains				;any new chains created after falling?
	LD A,(ChainExisting)
	OR A
	JR NZ,DestroyMarbles				;yes - repeat all

;end - stable position
;go to main game loop forgetting all calls and rets
	DI
	LD SP,65000
	EI
	JP MainLoop

	
;--------------------
AnimateDestruction
	LD D,3				;nr of animation frames
	
AnimateDestruction2
	PUSH DE
	CALL Sound3

;-------make marbles prepared to be destroyed dissapear------
	HALT
	HALT
	LD HL,DestructionTable
	
	LD B,0
AnimateDestruction5
	LD C,0

;one row
AnimateDestruction6
	LD A,(HL)						;marble status
	OR A
	JR Z,AnimateDestruction7		;not supposed to be destroyed
	
;blink marble supposed to be destroyed	
	PUSH BC
	PUSH HL
	CALL TransformBoardCoords		;after it HL-screen position
	LD BC,3*256+3					;size
	CALL DrawEmptySprite			;make marble dissapear
	POP HL
	POP BC

AnimateDestruction7
	INC HL							;next marble in the row
	INC C
	LD A,C
	CP 8
	JR NZ,AnimateDestruction6		;repeat until all row is done
	
	INC B
	LD A,B							;do all 8 rows
	CP 8
	JR NZ,AnimateDestruction5
	
;-------make marbles appear again--------
	HALT
	HALT	
	CALL DrawBoard
	
	POP DE
	DEC D
	JR NZ,AnimateDestruction2

	RET
	
	
	
;-----------------------------
Explosions
;numbers in table:
;0 - field not destroyed
;1 - marble destroyed through making a chain
;2 - diamond marked to be destroyed (will turn into 4 after exploding)
;3 - neighbour area destroyed by diamond
		;(if there is a diamond in this area, the field status will change from 3 to 2 later)
;4 - diamond that already exploded, another explosion won't change its status


;find if any of destroyed marbles are diamonds
	XOR A
	LD (ExplosionHappened_AtAll),A

Explosions1	
	LD BC,64*256+0
	LD DE,BoardData
	LD HL,DestructionTable
		
Explosions2
	LD A,(HL)
	OR A					;is marble at current position supposed to be destroyed, no->do nothing
	JR Z,Explosions3
	CP 4					;is it a diamond that already exploded -> do nothing
	JR Z,Explosions3
	
	
	LD A,(DE)				;is marble at current position a diamond (nr 9-15), no->do nothing
	CP 8
	JR C,Explosions3
	
	LD A,2				;mark diamond to be destroyed
	LD (HL),A			;2 means that it is diamond ready to explode
	
	LD C,1					;mark with C register that explosion will happen
	
Explosions3
	INC HL					
	INC DE
	DJNZ Explosions2

	LD A,C					;any explosions in this iteration?
	OR A
	JR Z,Explosions10		;no, end
;checking if any diamonds were found to explode in current go
;(when running this routine for the first time, they happened for sure
;but this code will be run several times as explosions may cause more explosions
;however the chain of explosions will end sooner or later)
	
	
	

;make explosions (mark exploding areas)
Explosion4
	LD HL,DestructionTable
	LD B,0					;row

Explosion5
	LD C,0					;collumn

Explosion6	
	LD A,(HL)				;check position
	CP 2
	CALL Z,MakeExplosion	;mark fields around diamond is ready for explosion
	
	INC HL					;next position in a table (no matter if we go one collumn right
							;or from left edge of row to left edge of one row down)
	INC C
	LD A,C
	CP 8
	JR NZ,Explosion6		;do one row, 8 fields
	
	INC B					;next row
	LD A,B
	CP 8
	JR NZ,Explosion5		;do 8 rows
		
	
;search for diamonds in area of explosions, they will make new explosions
	LD B,64
	LD DE,BoardData
	LD HL,DestructionTable
Explosions7	
	LD A,(HL)
	CP 3
	JR NZ,Explosions8			;field not in explosion area, do nothing
	LD A,(DE)
	CP 8
	JR C,Explosions8			;field does not have diamond, do nothing

	LD A,2						;set field status as diamond ready to be destroyed
	LD (HL),A					;(which will happen in next go)
	

Explosions8
	INC HL						;next field on board
	INC DE
	DJNZ Explosions7			;check all 64 fields on the board
	
	JR Explosions1				;repeat until no more explosions are found

	
;end of finding explosions, now animate them
Explosions10
	LD A, (ExplosionHappened_AtAll)
	OR A
	RET Z						;cehck if any explosions hapned at all, no -> finish

	CALL AnimateExplosions
	RET	
	

;-------
MakeExplosion

	PUSH BC						;store position in the table
	LD A,1
	LD (ExplosionHappened_AtAll),A
	
	LD A,4
	LD (HL),A					;change diamond status from "read to explode" to "already exploded"

;mark fields around diamond
	LD DE,8

;row above
	LD A,B
	OR A
	JR Z,MakeExplosion2			;don't mark row above diamond if diamond is in top row of board
	
	SBC HL,DE					;go to top row
	DEC B
	CALL MakeExplosion_MarkOneRow
	INC B
	ADD HL,DE

;row middle
MakeExplosion2
	CALL MakeExplosion_MarkOneRow		;always possible
	
;row below
	LD A,B
	CP 7
	JR Z,MakeExplosion3			;don't mark row below diamond if diamond is in bottom row of board
	INC B
	ADD HL,DE
	CALL MakeExplosion_MarkOneRow
	OR A
	SUB HL,DE
	
MakeExplosion3
	POP BC
	RET
	
;-----------	
MakeExplosion_MarkOneRow
;collumn left
	LD A,C
	OR A
	JR Z,MakeExplosion_MarkOneRow2		;skip if diamond on left edge of board
	DEC HL
	CALL MarkOneField
	INC HL
	
;collumn middle	
MakeExplosion_MarkOneRow2
	CALL MarkOneField					;mark middle field in a row, always possible
	
;collumn right	
	LD A,C
	CP 7
	RET Z								;end if diamond on right edge of board
	
	INC HL	
	CALL MarkOneField
	DEC HL
	RET	
	
;-----------
MarkOneField
	LD A,(HL)
	CP 2								;if in exploding area there is a diamond
	RET Z								;which exploded or is ready to explode
	CP 4								;don't change the field status
	RET Z
	
	LD A,3								;otherwise mark it as explosion area
	LD (HL),A
	RET
;----
ExplosionHappened_AtAll		.BYTE 0		;did any explosion happened at all
	
;---------------------------------------
AnimateExplosions
	CALL ExplosionSound

	LD B,40
	LD A,1
	LD (ExplosionColor),A
	
	
AnimateExplosions1	
	HALT
	PUSH BC

	LD HL,DestructionTable
	LD B,0					;row

AnimateExplosions2
	LD C,0					;collumn

AnimateExplosions3
	LD A,(HL)				;check field status
	CP 0					;field not destroyed
	JR Z,AnimateExplosions5	
	CP 1					;field destroyed by normal chain connecting, not diamond
	JR Z,AnimateExplosions5	
	
;draw explosion on this field
	PUSH BC					;store row and collumn
	PUSH HL					;store destruction table index
	
	CALL TransformBoardCoords	;HL - destroyed field position
	PUSH HL
	LD DE,DestroyedMarble
	LD BC,3*256+3
	CALL DrawSprite				;draw explosion sprite
	POP HL

	CALL GraphToAttr
	LD A,(ExplosionColor)
	LD BC,3*256+3
	CALL AttrBlock				;draw explosion color
	
	POP HL
	POP BC
	

AnimateExplosions5	
	INC HL					;next position in a row
	INC C					;next collumn
	LD A,C
	CP 8
	JR NZ,AnimateExplosions3	;do one row, 8 fields
	
	INC B						;next row
	LD A,B
	CP 8
	JR NZ,AnimateExplosions2	;do 8 rows
	
	POP BC					;color cycling
	LD A,(ExplosionColor)
	INC A
	CP 8
	JR NZ,AnimateExplosions6
	LD A,1
	
AnimateExplosions6
	LD (ExplosionColor),A
	DJNZ AnimateExplosions1
	RET	

ExplosionColor	.BYTE 0	



;----------------------
EraseMarbles
	LD HL,BoardData
	LD DE,DestructionTable
	LD BC,64*256+0			
	
EraseMarbles2
	LD A,(DE)						;should marble be destroyed?
	OR A
	JR Z,EraseMarbles3				;no
	
	LD (HL),C						;destroy marble - write 0 (empty) to its position

EraseMarbles3
	INC HL							;next marble data
	INC DE

	DJNZ EraseMarbles2
	RET
	
	
